@@ -10,20 +10,20 @@ from typing import (  # pylint: disable=unused-import
 
 from ._shared import sign_string, url_quote
 from ._shared.constants import X_MS_VERSION
-from ._shared.models import Services
+from ._shared.models import Services, UserDelegationKey
 from ._shared.shared_access_signature import SharedAccessSignature, _SharedAccessHelper, \
     QueryStringConstants
 
 if TYPE_CHECKING:
     from datetime import datetime
-    from . import (
+    from ..blob import (
         ResourceTypes,
         AccountSasPermissions,
-        UserDelegationKey,
         ContainerSasPermissions,
         BlobSasPermissions
     )
 
+
 class BlobQueryStringConstants(object):
     SIGNED_TIMESTAMP = 'snapshot'
 
@@ -50,11 +50,11 @@ class BlobSharedAccessSignature(SharedAccessSignature):
         super(BlobSharedAccessSignature, self).__init__(account_name, account_key, x_ms_version=X_MS_VERSION)
         self.user_delegation_key = user_delegation_key
 
-    def generate_blob(self, container_name, blob_name, snapshot=None, permission=None,
+    def generate_blob(self, container_name, blob_name, snapshot=None, version_id=None, permission=None,
                       expiry=None, start=None, policy_id=None, ip=None, protocol=None,
                       cache_control=None, content_disposition=None,
                       content_encoding=None, content_language=None,
-                      content_type=None):
+                      content_type=None, **kwargs):
         '''
         Generates a shared access signature for the blob or one of its snapshots.
         Use the returned signature with the sas_token parameter of any BlobService.
@@ -66,13 +66,14 @@ class BlobSharedAccessSignature(SharedAccessSignature):
         :param str snapshot:
             The snapshot parameter is an opaque DateTime value that,
             when present, specifies the blob snapshot to grant permission.
-        :param BlobSasPermissions permission:
+        :param permission:
             The permissions associated with the shared access signature. The
             user is restricted to operations allowed by the permissions.
-            Permissions must be ordered read, write, delete, list.
+            Permissions must be ordered racwdxytmei.
             Required unless an id is given referencing a stored access policy
             which contains this field. This field must be omitted if it has been
             specified in an associated stored access policy.
+        :type permission: str or BlobSasPermissions
         :param expiry:
             The time at which the shared access signature becomes invalid.
             Required unless an id is given referencing a stored access policy
@@ -122,11 +123,18 @@ class BlobSharedAccessSignature(SharedAccessSignature):
         sas = _BlobSharedAccessHelper()
         sas.add_base(permission, expiry, start, ip, protocol, self.x_ms_version)
         sas.add_id(policy_id)
-        sas.add_resource('b' if snapshot is None else 'bs')
-        sas.add_timestamp(snapshot)
+
+        resource = 'bs' if snapshot else 'b'
+        resource = 'bv' if version_id else resource
+        resource = 'd' if kwargs.pop("is_directory", None) else resource
+        sas.add_resource(resource)
+
+        sas.add_timestamp(snapshot or version_id)
         sas.add_override_response_headers(cache_control, content_disposition,
                                           content_encoding, content_language,
                                           content_type)
+        sas.add_encryption_scope(**kwargs)
+        sas.add_info_for_hns_account(**kwargs)
         sas.add_resource_signature(self.account_name, self.account_key, resource_path,
                                    user_delegation_key=self.user_delegation_key)
 
@@ -136,20 +144,21 @@ class BlobSharedAccessSignature(SharedAccessSignature):
                            start=None, policy_id=None, ip=None, protocol=None,
                            cache_control=None, content_disposition=None,
                            content_encoding=None, content_language=None,
-                           content_type=None):
+                           content_type=None, **kwargs):
         '''
         Generates a shared access signature for the container.
         Use the returned signature with the sas_token parameter of any BlobService.
 
         :param str container_name:
             Name of container.
-        :param ContainerSasPermissions permission:
+        :param permission:
             The permissions associated with the shared access signature. The
             user is restricted to operations allowed by the permissions.
-            Permissions must be ordered read, write, delete, list.
+            Permissions must be ordered racwdxyltfmei.
             Required unless an id is given referencing a stored access policy
             which contains this field. This field must be omitted if it has been
             specified in an associated stored access policy.
+        :type permission: str or ContainerSasPermissions
         :param expiry:
             The time at which the shared access signature becomes invalid.
             Required unless an id is given referencing a stored access policy
@@ -201,6 +210,8 @@ class BlobSharedAccessSignature(SharedAccessSignature):
         sas.add_override_response_headers(cache_control, content_disposition,
                                           content_encoding, content_language,
                                           content_type)
+        sas.add_encryption_scope(**kwargs)
+        sas.add_info_for_hns_account(**kwargs)
         sas.add_resource_signature(self.account_name, self.account_key, container_name,
                                    user_delegation_key=self.user_delegation_key)
         return sas.get_token()
@@ -211,6 +222,12 @@ class _BlobSharedAccessHelper(_SharedAccessHelper):
     def add_timestamp(self, timestamp):
         self._add_query(BlobQueryStringConstants.SIGNED_TIMESTAMP, timestamp)
 
+    def add_info_for_hns_account(self, **kwargs):
+        self._add_query(QueryStringConstants.SIGNED_DIRECTORY_DEPTH, kwargs.pop('sdd', None))
+        self._add_query(QueryStringConstants.SIGNED_AUTHORIZED_OID, kwargs.pop('preauthorized_agent_object_id', None))
+        self._add_query(QueryStringConstants.SIGNED_UNAUTHORIZED_OID, kwargs.pop('agent_object_id', None))
+        self._add_query(QueryStringConstants.SIGNED_CORRELATION_ID, kwargs.pop('correlation_id', None))
+
     def get_value_to_append(self, query):
         return_value = self.query_dict.get(query) or ''
         return return_value + '\n'
@@ -244,7 +261,10 @@ class _BlobSharedAccessHelper(_SharedAccessHelper):
                  self.get_value_to_append(QueryStringConstants.SIGNED_KEY_START) +
                  self.get_value_to_append(QueryStringConstants.SIGNED_KEY_EXPIRY) +
                  self.get_value_to_append(QueryStringConstants.SIGNED_KEY_SERVICE) +
-                 self.get_value_to_append(QueryStringConstants.SIGNED_KEY_VERSION))
+                 self.get_value_to_append(QueryStringConstants.SIGNED_KEY_VERSION) +
+                 self.get_value_to_append(QueryStringConstants.SIGNED_AUTHORIZED_OID) +
+                 self.get_value_to_append(QueryStringConstants.SIGNED_UNAUTHORIZED_OID) +
+                 self.get_value_to_append(QueryStringConstants.SIGNED_CORRELATION_ID))
         else:
             string_to_sign += self.get_value_to_append(QueryStringConstants.SIGNED_IDENTIFIER)
 
@@ -254,6 +274,7 @@ class _BlobSharedAccessHelper(_SharedAccessHelper):
              self.get_value_to_append(QueryStringConstants.SIGNED_VERSION) +
              self.get_value_to_append(QueryStringConstants.SIGNED_RESOURCE) +
              self.get_value_to_append(BlobQueryStringConstants.SIGNED_TIMESTAMP) +
+             self.get_value_to_append(QueryStringConstants.SIGNED_ENCRYPTION_SCOPE) +
              self.get_value_to_append(QueryStringConstants.SIGNED_CACHE_CONTROL) +
              self.get_value_to_append(QueryStringConstants.SIGNED_CONTENT_DISPOSITION) +
              self.get_value_to_append(QueryStringConstants.SIGNED_CONTENT_ENCODING) +
@@ -328,6 +349,8 @@ def generate_account_sas(
         restricts the request to those IP addresses.
     :keyword str protocol:
         Specifies the protocol permitted for a request made. The default value is https.
+    :keyword str encryption_scope:
+        Specifies the encryption scope for a request made so that all write operations will be service encrypted.
     :return: A Shared Access Signature (sas) token.
     :rtype: str
 
@@ -385,7 +408,7 @@ def generate_container_sas(
     :param permission:
         The permissions associated with the shared access signature. The
         user is restricted to operations allowed by the permissions.
-        Permissions must be ordered read, write, delete, list.
+        Permissions must be ordered racwdxyltfmei.
         Required unless an id is given referencing a stored access policy
         which contains this field. This field must be omitted if it has been
         specified in an associated stored access policy.
@@ -432,6 +455,8 @@ def generate_container_sas(
     :keyword str content_type:
         Response header value for Content-Type when resource is accessed
         using this shared access signature.
+    :keyword str encryption_scope:
+        Specifies the encryption scope for a request made so that all write operations will be service encrypted.
     :return: A Shared Access Signature (sas) token.
     :rtype: str
 
@@ -446,7 +471,8 @@ def generate_container_sas(
     """
     if not user_delegation_key and not account_key:
         raise ValueError("Either user_delegation_key or account_key must be provided.")
-
+    if isinstance(account_key, UserDelegationKey):
+        user_delegation_key = account_key
     if user_delegation_key:
         sas = BlobSharedAccessSignature(account_name, user_delegation_key=user_delegation_key)
     else:
@@ -501,7 +527,7 @@ def generate_blob_sas(
     :param permission:
         The permissions associated with the shared access signature. The
         user is restricted to operations allowed by the permissions.
-        Permissions must be ordered read, write, delete, list.
+        Permissions must be ordered racwdxytmei.
         Required unless an id is given referencing a stored access policy
         which contains this field. This field must be omitted if it has been
         specified in an associated stored access policy.
@@ -531,6 +557,11 @@ def generate_blob_sas(
         or address range specified on the SAS token, the request is not authenticated.
         For example, specifying ip=168.1.5.65 or ip=168.1.5.60-168.1.5.70 on the SAS
         restricts the request to those IP addresses.
+    :keyword str version_id:
+        An optional blob version ID. This parameter is only for versioning enabled account
+
+        .. versionadded:: 12.4.0
+            This keyword argument was introduced in API version '2019-12-12'.
     :keyword str protocol:
         Specifies the protocol permitted for a request made. The default value is https.
     :keyword str cache_control:
@@ -548,12 +579,18 @@ def generate_blob_sas(
     :keyword str content_type:
         Response header value for Content-Type when resource is accessed
         using this shared access signature.
+    :keyword str encryption_scope:
+        Specifies the encryption scope for a request made so that all write operations will be service encrypted.
     :return: A Shared Access Signature (sas) token.
     :rtype: str
     """
     if not user_delegation_key and not account_key:
         raise ValueError("Either user_delegation_key or account_key must be provided.")
-
+    if isinstance(account_key, UserDelegationKey):
+        user_delegation_key = account_key
+    version_id = kwargs.pop('version_id', None)
+    if version_id and snapshot:
+        raise ValueError("snapshot and version_id cannot be set at the same time.")
     if user_delegation_key:
         sas = BlobSharedAccessSignature(account_name, user_delegation_key=user_delegation_key)
     else:
@@ -562,6 +599,7 @@ def generate_blob_sas(
         container_name,
         blob_name,
         snapshot=snapshot,
+        version_id=version_id,
         permission=permission,
         expiry=expiry,
         start=start,
