@@ -4,7 +4,9 @@
 # license information.
 # --------------------------------------------------------------------------
 from io import BytesIO
-from typing import Any
+from typing import ( # pylint: disable=unused-import
+    Any, AnyStr, Dict, IO, Iterable, Optional, Type, TypeVar, Union,
+    TYPE_CHECKING)
 
 try:
     from urllib.parse import quote, unquote
@@ -23,10 +25,16 @@ from ._upload_helper import upload_datalake_file
 from ._download import StorageStreamDownloader
 from ._path_client import PathClient
 from ._serialize import get_mod_conditions, get_path_http_headers, get_access_conditions, add_metadata_headers, \
-    convert_datetime_to_rfc1123
+    convert_datetime_to_rfc1123, get_cpk_info
 from ._deserialize import process_storage_error, deserialize_file_properties
 from ._models import FileProperties, DataLakeFileQueryError
 
+if TYPE_CHECKING:
+    from datetime import datetime
+    from ._models import ContentSettings
+
+ClassType = TypeVar("ClassType")
+
 
 class DataLakeFileClient(PathClient):
     """A client to interact with the DataLake file, even if the file may not yet exist.
@@ -49,10 +57,15 @@ class DataLakeFileClient(PathClient):
     :param credential:
         The credentials with which to authenticate. This is optional if the
         account URL already has a SAS token. The value can be a SAS token string,
-        an instance of a AzureSasCredential from azure.core.credentials, an account
-        shared access key, or an instance of a TokenCredentials class from azure.identity.
+        an instance of a AzureSasCredential or AzureNamedKeyCredential from azure.core.credentials,
+        an account shared access key, or an instance of a TokenCredentials class from azure.identity.
         If the resource URI already contains a SAS token, this will be ignored in favor of an explicit credential
         - except in the case of AzureSasCredential, where the conflicting SAS tokens will raise a ValueError.
+        If using an instance of AzureNamedKeyCredential, "name" should be the storage account name, and "key"
+        should be the storage account key.
+    :keyword str api_version:
+        The Storage API version to use for requests. Default value is the most recent service version that is
+        compatible with the current SDK. Setting to an older version may result in reduced feature compatibility.
 
     .. admonition:: Example:
 
@@ -67,7 +80,7 @@ class DataLakeFileClient(PathClient):
         self, account_url,  # type: str
         file_system_name,  # type: str
         file_path,  # type: str
-        credential=None,  # type: Optional[Any]
+        credential=None,  # type: Optional[Union[str, Dict[str, str], AzureNamedKeyCredential, AzureSasCredential, "TokenCredential"]] # pylint: disable=line-too-long
         **kwargs  # type: Any
     ):
         # type: (...) -> None
@@ -76,12 +89,13 @@ class DataLakeFileClient(PathClient):
 
     @classmethod
     def from_connection_string(
-            cls, conn_str,  # type: str
+            cls,  # type: Type[ClassType]
+            conn_str,  # type: str
             file_system_name,  # type: str
             file_path,  # type: str
-            credential=None,  # type: Optional[Any]
+            credential=None,  # type: Optional[Union[str, Dict[str, str], AzureNamedKeyCredential, AzureSasCredential, "TokenCredential"]] # pylint: disable=line-too-long
             **kwargs  # type: Any
-        ):  # type: (...) -> DataLakeFileClient
+        ):  # type: (...) -> ClassType
         """
         Create DataLakeFileClient from a Connection String.
 
@@ -97,9 +111,11 @@ class DataLakeFileClient(PathClient):
             The credentials with which to authenticate. This is optional if the
             account URL already has a SAS token, or the connection string already has shared
             access key values. The value can be a SAS token string,
-            an instance of a AzureSasCredential from azure.core.credentials, an account shared access
-            key, or an instance of a TokenCredentials class from azure.identity.
+            an instance of a AzureSasCredential or AzureNamedKeyCredential from azure.core.credentials,
+            an account shared access key, or an instance of a TokenCredentials class from azure.identity.
             Credentials provided here will take precedence over those in the connection string.
+            If using an instance of AzureNamedKeyCredential, "name" should be the storage account name, and "key"
+            should be the storage account key.
         :return a DataLakeFileClient
         :rtype ~azure.storage.filedatalake.DataLakeFileClient
         """
@@ -132,6 +148,31 @@ class DataLakeFileClient(PathClient):
             For example, if p is 0777 and u is 0057, then the resulting permission is 0720.
             The default permission is 0777 for a directory and 0666 for a file. The default umask is 0027.
             The umask must be specified in 4-digit octal notation (e.g. 0766).
+        :keyword str owner:
+            The owner of the file or directory.
+        :keyword str group:
+            The owning group of the file or directory.
+        :keyword str acl:
+            Sets POSIX access control rights on files and directories. The value is a
+            comma-separated list of access control entries. Each access control entry (ACE) consists of a
+            scope, a type, a user or group identifier, and permissions in the format
+            "[scope:][type]:[id]:[permissions]".
+        :keyword str lease_id:
+            Proposed lease ID, in a GUID string format. The DataLake service returns
+            400 (Invalid request) if the proposed lease ID is not in the correct format.
+        :keyword int lease_duration:
+            Specifies the duration of the lease, in seconds, or negative one
+            (-1) for a lease that never expires. A non-infinite lease can be
+            between 15 and 60 seconds. A lease duration cannot be changed
+            using renew or change.
+        :keyword expires_on:
+            The time to set the file to expiry.
+            If the type of expires_on is an int, expiration time will be set
+            as the number of milliseconds elapsed from creation time.
+            If the type of expires_on is datetime, expiration time will be set
+            absolute to the time provided. If no time zone info is provided, this
+            will be interpreted as UTC.
+        :paramtype expires_on: datetime or int
         :keyword str permissions:
             Optional and only valid if Hierarchical Namespace
             is enabled for the account. Sets POSIX access permissions for the file
@@ -156,6 +197,9 @@ class DataLakeFileClient(PathClient):
             and act according to the condition specified by the `match_condition` parameter.
         :keyword ~azure.core.MatchConditions match_condition:
             The match condition to use upon the etag.
+        :keyword ~azure.storage.filedatalake.CustomerProvidedEncryptionKey cpk:
+            Encrypts the data on the service-side with the given key.
+            Use of customer-provided keys must be done over HTTPS.
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
         :return: response dict (Etag and last modified).
@@ -238,6 +282,10 @@ class DataLakeFileClient(PathClient):
             and act according to the condition specified by the `match_condition` parameter.
         :keyword ~azure.core.MatchConditions match_condition:
             The match condition to use upon the etag.
+        :keyword ~azure.storage.filedatalake.CustomerProvidedEncryptionKey cpk:
+            Decrypts the data on the service-side with the given key.
+            Use of customer-provided keys must be done over HTTPS.
+            Required if the file was created with a customer-provided key.
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
         :rtype: FileProperties
@@ -256,7 +304,7 @@ class DataLakeFileClient(PathClient):
     def set_file_expiry(self, expiry_options,  # type: str
                         expires_on=None,   # type: Optional[Union[datetime, int]]
                         **kwargs):
-        # type: (str, Optional[Union[datetime, int]], **Any) -> None
+        # type: (...) -> None
         """Sets the time a file will expire and be deleted.
 
         :param str expiry_options:
@@ -278,7 +326,7 @@ class DataLakeFileClient(PathClient):
             .set_expiry(expiry_options, expires_on=expires_on, **kwargs)  # pylint: disable=protected-access
 
     def _upload_options(  # pylint:disable=too-many-statements
-            self, data,  # type: Union[Iterable[AnyStr], IO[AnyStr]]
+            self, data,  # type: Union[bytes, str, Iterable[AnyStr], IO[AnyStr]]
             length=None,  # type: Optional[int]
             **kwargs
         ):
@@ -309,6 +357,7 @@ class DataLakeFileClient(PathClient):
         kwargs['properties'] = add_metadata_headers(metadata)
         kwargs['lease_access_conditions'] = get_access_conditions(kwargs.pop('lease', None))
         kwargs['modified_access_conditions'] = get_mod_conditions(kwargs)
+        kwargs['cpk_info'] = get_cpk_info(self.scheme, kwargs)
 
         if content_settings:
             kwargs['path_http_headers'] = get_path_http_headers(content_settings)
@@ -322,7 +371,7 @@ class DataLakeFileClient(PathClient):
 
         return kwargs
 
-    def upload_data(self, data,  # type: Union[AnyStr, Iterable[AnyStr], IO[AnyStr]]
+    def upload_data(self, data,  # type: Union[bytes, str, Iterable[AnyStr], IO[AnyStr]]
                     length=None,  # type: Optional[int]
                     overwrite=False,  # type: Optional[bool]
                     **kwargs):
@@ -380,6 +429,9 @@ class DataLakeFileClient(PathClient):
             and act according to the condition specified by the `match_condition` parameter.
         :keyword ~azure.core.MatchConditions match_condition:
             The match condition to use upon the etag.
+        :keyword ~azure.storage.filedatalake.CustomerProvidedEncryptionKey cpk:
+            Encrypts the data on the service-side with the given key.
+            Use of customer-provided keys must be done over HTTPS.
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
         :keyword int chunk_size:
@@ -395,8 +447,14 @@ class DataLakeFileClient(PathClient):
         return upload_datalake_file(**options)
 
     @staticmethod
-    def _append_data_options(data, offset, length=None, **kwargs):
-        # type: (Optional[ContentSettings], Optional[Dict[str, str]], **Any) -> Dict[str, Any]
+    def _append_data_options(
+            data, # type: Union[bytes, str, Iterable[AnyStr], IO[AnyStr]]
+            offset, # type: int
+            scheme, # type: str
+            length=None, # type: Optional[int]
+            **kwargs
+        ):
+        # type: (...) -> Dict[str, Any]
 
         if isinstance(data, six.text_type):
             data = data.encode(kwargs.pop('encoding', 'UTF-8'))  # type: ignore
@@ -408,6 +466,7 @@ class DataLakeFileClient(PathClient):
             data = data[:length]
 
         access_conditions = get_access_conditions(kwargs.pop('lease', None))
+        cpk_info = get_cpk_info(scheme, kwargs)
 
         options = {
             'body': data,
@@ -415,12 +474,13 @@ class DataLakeFileClient(PathClient):
             'content_length': length,
             'lease_access_conditions': access_conditions,
             'validate_content': kwargs.pop('validate_content', False),
+            'cpk_info': cpk_info,
             'timeout': kwargs.pop('timeout', None),
             'cls': return_response_headers}
         options.update(kwargs)
         return options
 
-    def append_data(self, data,  # type: Union[AnyStr, Iterable[AnyStr], IO[AnyStr]]
+    def append_data(self, data,  # type: Union[bytes, str, Iterable[AnyStr], IO[AnyStr]]
                     offset,  # type: int
                     length=None,  # type: Optional[int]
                     **kwargs):
@@ -430,6 +490,8 @@ class DataLakeFileClient(PathClient):
         :param data: Content to be appended to file
         :param offset: start position of the data to be appended to.
         :param length: Size of the data in bytes.
+        :keyword bool flush:
+            If true, will commit the data after it is appended.
         :keyword bool validate_content:
             If true, calculates an MD5 hash of the block content. The storage
             service checks the hash of the content that has arrived
@@ -441,6 +503,9 @@ class DataLakeFileClient(PathClient):
             Required if the file has an active lease. Value can be a DataLakeLeaseClient object
             or the lease ID as a string.
         :paramtype lease: ~azure.storage.filedatalake.DataLakeLeaseClient or str
+        :keyword ~azure.storage.filedatalake.CustomerProvidedEncryptionKey cpk:
+            Encrypts the data on the service-side with the given key.
+            Use of customer-provided keys must be done over HTTPS.
         :return: dict of the response header
 
         .. admonition:: Example:
@@ -453,8 +518,9 @@ class DataLakeFileClient(PathClient):
                 :caption: Append data to the file.
         """
         options = self._append_data_options(
-            data,
-            offset,
+            data=data,
+            offset=offset,
+            scheme=self.scheme,
             length=length,
             **kwargs)
         try:
@@ -463,8 +529,14 @@ class DataLakeFileClient(PathClient):
             process_storage_error(error)
 
     @staticmethod
-    def _flush_data_options(offset, content_settings=None, retain_uncommitted_data=False, **kwargs):
-        # type: (Optional[ContentSettings], Optional[Dict[str, str]], **Any) -> Dict[str, Any]
+    def _flush_data_options(
+            offset, # type: int
+            scheme, # type: str
+            content_settings=None, # type: Optional[ContentSettings]
+            retain_uncommitted_data=False, # type: Optional[bool]
+            **kwargs
+        ):
+        # type: (...) -> Dict[str, Any]
 
         access_conditions = get_access_conditions(kwargs.pop('lease', None))
         mod_conditions = get_mod_conditions(kwargs)
@@ -473,6 +545,8 @@ class DataLakeFileClient(PathClient):
         if content_settings:
             path_http_headers = get_path_http_headers(content_settings)
 
+        cpk_info = get_cpk_info(scheme, kwargs)
+
         options = {
             'position': offset,
             'content_length': 0,
@@ -481,6 +555,7 @@ class DataLakeFileClient(PathClient):
             'close': kwargs.pop('close', False),
             'lease_access_conditions': access_conditions,
             'modified_access_conditions': mod_conditions,
+            'cpk_info': cpk_info,
             'timeout': kwargs.pop('timeout', None),
             'cls': return_response_headers}
         options.update(kwargs)
@@ -534,6 +609,9 @@ class DataLakeFileClient(PathClient):
             and act according to the condition specified by the `match_condition` parameter.
         :keyword ~azure.core.MatchConditions match_condition:
             The match condition to use upon the etag.
+        :keyword ~azure.storage.filedatalake.CustomerProvidedEncryptionKey cpk:
+            Encrypts the data on the service-side with the given key.
+            Use of customer-provided keys must be done over HTTPS.
         :return: response header in dict
 
         .. admonition:: Example:
@@ -547,6 +625,7 @@ class DataLakeFileClient(PathClient):
         """
         options = self._flush_data_options(
             offset,
+            self.scheme,
             retain_uncommitted_data=retain_uncommitted_data, **kwargs)
         try:
             return self._client.path.flush_data(**options)
@@ -586,6 +665,10 @@ class DataLakeFileClient(PathClient):
             and act according to the condition specified by the `match_condition` parameter.
         :keyword ~azure.core.MatchConditions match_condition:
             The match condition to use upon the etag.
+        :keyword ~azure.storage.filedatalake.CustomerProvidedEncryptionKey cpk:
+            Decrypts the data on the service-side with the given key.
+            Use of customer-provided keys must be done over HTTPS.
+            Required if the file was created with a Customer-Provided Key.
         :keyword int max_concurrency:
             The number of parallel connections with which to download.
         :keyword int timeout:
@@ -612,7 +695,7 @@ class DataLakeFileClient(PathClient):
         """
         Returns True if a file exists and returns False otherwise.
 
-        :kwarg int timeout:
+        :keyword int timeout:
             The timeout parameter is expressed in seconds.
         :returns: boolean
         """
@@ -699,9 +782,7 @@ class DataLakeFileClient(PathClient):
             "{}://{}".format(self.scheme, self.primary_hostname), new_file_system, file_path=new_path,
             credential=self._raw_credential or new_file_sas,
             _hosts=self._hosts, _configuration=self._config, _pipeline=self._pipeline,
-            _location_mode=self._location_mode, require_encryption=self.require_encryption,
-            key_encryption_key=self.key_encryption_key,
-            key_resolver_function=self.key_resolver_function
+            _location_mode=self._location_mode
         )
         new_file_client._rename_path(  # pylint: disable=protected-access
             '/{}/{}{}'.format(quote(unquote(self.file_system_name)),
@@ -759,6 +840,10 @@ class DataLakeFileClient(PathClient):
             and act according to the condition specified by the `match_condition` parameter.
         :keyword ~azure.core.MatchConditions match_condition:
             The match condition to use upon the etag.
+        :keyword ~azure.storage.filedatalake.CustomerProvidedEncryptionKey cpk:
+            Decrypts the data on the service-side with the given key.
+            Use of customer-provided keys must be done over HTTPS.
+            Required if the file was created with a Customer-Provided Key.
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
         :returns: A streaming object (DataLakeFileQueryReader)
