@@ -8,40 +8,47 @@ import sys
 import threading
 import warnings
 from io import BytesIO
-from typing import Iterator
+from typing import (
+    Any, Callable, Generator, IO, Iterator, Optional, Tuple,
+    TYPE_CHECKING
+)
 
 from azure.core.exceptions import HttpResponseError, ResourceModifiedError
 from azure.core.tracing.common import with_current_context
 from ._shared.request_handlers import validate_and_format_range_headers
-from ._shared.response_handlers import process_storage_error, parse_length_from_content_range
+from ._shared.response_handlers import parse_length_from_content_range, process_storage_error
+
+if TYPE_CHECKING:
+    from ._generated.operations import FileOperations
+    from ._models import FileProperties
+    from ._shared.models import StorageConfiguration
 
 
-def process_content(data):
+def process_content(data: Any) -> bytes:
     if data is None:
         raise ValueError("Response cannot be None.")
 
     try:
         return b"".join(list(data))
     except Exception as error:
-        raise HttpResponseError(message="Download stream interrupted.", response=data.response, error=error)
+        raise HttpResponseError(message="Download stream interrupted.", response=data.response, error=error) from error
 
 
 class _ChunkDownloader(object):  # pylint: disable=too-many-instance-attributes
     def __init__(
-        self,
-        client=None,
-        total_size=None,
-        chunk_size=None,
-        current_progress=None,
-        start_range=None,
-        end_range=None,
-        stream=None,
-        parallel=None,
-        validate_content=None,
-        progress_hook=None,
-        etag=None,
-        **kwargs
-    ):
+        self, client: "FileOperations",
+        total_size: int,
+        chunk_size: int,
+        current_progress: int,
+        start_range: int,
+        end_range: int,
+        validate_content: bool,
+        etag: str,
+        stream: Any = None,
+        parallel: Optional[int] = None,
+        progress_hook: Optional[Callable[[int, Optional[int]], None]] = None,
+        **kwargs: Any
+    ) -> None:
         self.client = client
         self.etag = etag
         # Information on the download range/chunk size
@@ -58,7 +65,7 @@ class _ChunkDownloader(object):  # pylint: disable=too-many-instance-attributes
 
         # For a parallel download, the stream is always seekable, so we note down the current position
         # in order to seek to the right place when out-of-order chunks come in
-        self.stream_start = stream.tell() if parallel else None
+        self.stream_start = stream.tell() if parallel else 0
 
         # Download progress so far
         self.progress_total = current_progress
@@ -67,20 +74,20 @@ class _ChunkDownloader(object):  # pylint: disable=too-many-instance-attributes
         self.validate_content = validate_content
         self.request_options = kwargs
 
-    def _calculate_range(self, chunk_start):
+    def _calculate_range(self, chunk_start: int) -> Tuple[int, int]:
         if chunk_start + self.chunk_size > self.end_index:
             chunk_end = self.end_index
         else:
             chunk_end = chunk_start + self.chunk_size
         return chunk_start, chunk_end
 
-    def get_chunk_offsets(self):
+    def get_chunk_offsets(self) -> Generator[int, None, None]:
         index = self.start_index
         while index < self.end_index:
             yield index
             index += self.chunk_size
 
-    def process_chunk(self, chunk_start):
+    def process_chunk(self, chunk_start: int) -> None:
         chunk_start, chunk_end = self._calculate_range(chunk_start)
         chunk_data = self._download_chunk(chunk_start, chunk_end - 1)
         length = chunk_end - chunk_start
@@ -88,11 +95,11 @@ class _ChunkDownloader(object):  # pylint: disable=too-many-instance-attributes
             self._write_to_stream(chunk_data, chunk_start)
             self._update_progress(length)
 
-    def yield_chunk(self, chunk_start):
+    def yield_chunk(self, chunk_start: int) -> bytes:
         chunk_start, chunk_end = self._calculate_range(chunk_start)
         return self._download_chunk(chunk_start, chunk_end - 1)
 
-    def _update_progress(self, length):
+    def _update_progress(self, length: int) -> None:
         if self.progress_lock:
             with self.progress_lock:  # pylint: disable=not-context-manager
                 self.progress_total += length
@@ -102,7 +109,7 @@ class _ChunkDownloader(object):  # pylint: disable=too-many-instance-attributes
         if self.progress_hook:
             self.progress_hook(self.progress_total, self.total_size)
 
-    def _write_to_stream(self, chunk_data, chunk_start):
+    def _write_to_stream(self, chunk_data: bytes, chunk_start: int) -> None:
         if self.stream_lock:
             with self.stream_lock:  # pylint: disable=not-context-manager
                 self.stream.seek(self.stream_start + (chunk_start - self.start_index))
@@ -110,12 +117,13 @@ class _ChunkDownloader(object):  # pylint: disable=too-many-instance-attributes
         else:
             self.stream.write(chunk_data)
 
-    def _download_chunk(self, chunk_start, chunk_end):
+    def _download_chunk(self, chunk_start: int, chunk_end: int) -> bytes:
         range_header, range_validation = validate_and_format_range_headers(
             chunk_start, chunk_end, check_content_md5=self.validate_content
         )
 
         try:
+            response: Any = None
             _, response = self.client.download(
                 range=range_header,
                 range_get_content_md5=range_validation,
@@ -135,24 +143,23 @@ class _ChunkDownloader(object):  # pylint: disable=too-many-instance-attributes
 
 
 class _ChunkIterator(object):
-    """Async iterator for chunks in blob download stream."""
+    """Iterator for chunks in file download stream."""
 
-    def __init__(self, size, content, downloader, chunk_size):
+    def __init__(self, size: int, content: bytes, downloader: Optional[_ChunkDownloader], chunk_size: int) -> None:
         self.size = size
         self._chunk_size = chunk_size
         self._current_content = content
         self._iter_downloader = downloader
-        self._iter_chunks = None
-        self._complete = (size == 0)
+        self._iter_chunks: Optional[Generator[int, None, None]] = None
+        self._complete = size == 0
 
-    def __len__(self):
+    def __len__(self) -> int:
         return self.size
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator[bytes]:
         return self
 
-    def __next__(self):
-        """Iterate through responses."""
+    def __next__(self) -> bytes:
         if self._complete:
             raise StopIteration("Download complete")
         if not self._iter_downloader:
@@ -182,48 +189,45 @@ class _ChunkIterator(object):
 
     next = __next__  # Python 2 compatibility.
 
-    def _get_chunk_data(self):
+    def _get_chunk_data(self) -> bytes:
         chunk_data = self._current_content[: self._chunk_size]
         self._current_content = self._current_content[self._chunk_size:]
         return chunk_data
 
 
 class StorageStreamDownloader(object):  # pylint: disable=too-many-instance-attributes
-    """A streaming object to download from Azure Storage.
+    """A streaming object to download from Azure Storage."""
 
-    :ivar str name:
-        The name of the file being downloaded.
-    :ivar: str path:
-        The full path of the file.
-    :ivar str share:
-        The name of the share where the file is.
-    :ivar ~azure.storage.fileshare.FileProperties properties:
-        The properties of the file being downloaded. If only a range of the data is being
-        downloaded, this will be reflected in the properties.
-    :ivar int size:
-        The size of the total data in the stream. This will be the byte range if specified,
-        otherwise the total size of the file.
-    """
+    name: str
+    """The name of the file being downloaded."""
+    path: str
+    """The full path of the file."""
+    share: str
+    """The name of the share where the file is."""
+    properties: "FileProperties"
+    """The properties of the file being downloaded. If only a range of the data is being
+        downloaded, this will be reflected in the properties."""
+    size: int
+    """The size of the total data in the stream. This will be the byte range if specified,
+        otherwise the total size of the file."""
 
     def __init__(
-        self,
-        client=None,
-        config=None,
-        start_range=None,
-        end_range=None,
-        validate_content=None,
-        max_concurrency=1,
-        name=None,
-        path=None,
-        share=None,
-        encoding=None,
-        **kwargs
-    ):
+        self, client: "FileOperations" = None,  # type: ignore [assignment]
+        config: "StorageConfiguration" = None,  # type: ignore [assignment]
+        start_range: Optional[int] = None,
+        end_range: Optional[int] = None,
+        validate_content: bool = None,  # type: ignore [assignment]
+        max_concurrency: int = 1,
+        name: str = None,  # type: ignore [assignment]
+        path: str = None,  # type: ignore [assignment]
+        share: str = None,  # type: ignore [assignment]
+        encoding: Optional[str] = None,
+        **kwargs: Any
+    ) -> None:
         self.name = name
         self.path = path
         self.share = share
-        self.properties = None
-        self.size = None
+        self.size = 0
 
         self._client = client
         self._config = config
@@ -236,10 +240,10 @@ class StorageStreamDownloader(object):  # pylint: disable=too-many-instance-attr
         self._request_options = kwargs
         self._location_mode = None
         self._download_complete = False
-        self._current_content = None
-        self._file_size = None
+        self._current_content = b""
+        self._file_size = 0
         self._response = None
-        self._etag = None
+        self._etag = ""
 
         # The service only provides transactional MD5s for chunks under 4MB.
         # If validate_content is on, get only self.MAX_CHUNK_GET_SIZE for the first
@@ -247,8 +251,8 @@ class StorageStreamDownloader(object):  # pylint: disable=too-many-instance-attr
         self._first_get_size = (
             self._config.max_single_get_size if not self._validate_content else self._config.max_chunk_get_size
         )
-        initial_request_start = self._start_range if self._start_range is not None else 0
-        if self._end_range is not None and self._end_range - self._start_range < self._first_get_size:
+        initial_request_start = self._start_range or 0
+        if self._end_range is not None and self._end_range - initial_request_start < self._first_get_size:
             initial_request_end = self._end_range
         else:
             initial_request_end = initial_request_start + self._first_get_size - 1
@@ -266,23 +270,19 @@ class StorageStreamDownloader(object):  # pylint: disable=too-many-instance-attr
         self.properties.size = self.size
 
         # Overwrite the content range to the user requested range
-        self.properties.content_range = "bytes {0}-{1}/{2}".format(
-            self._start_range,
-            self._end_range,
-            self._file_size
-        )
+        self.properties.content_range = f"bytes {self._start_range}-{self._end_range}/{self._file_size}"
 
         # Overwrite the content MD5 as it is the MD5 for the last range instead
         # of the stored MD5
         # TODO: Set to the stored MD5 when the service returns this
-        self.properties.content_md5 = None
+        self.properties.content_md5 = None  # type: ignore [attr-defined]
 
         if self.size == 0:
             self._current_content = b""
         else:
             self._current_content = process_content(self._response)
 
-    def __len__(self):
+    def __len__(self) -> int:
         return self.size
 
     def _initial_request(self):
@@ -334,8 +334,8 @@ class StorageStreamDownloader(object):  # pylint: disable=too-many-instance-attr
                         download_stream_current=0,
                         **self._request_options
                     )
-                except HttpResponseError as error:
-                    process_storage_error(error)
+                except HttpResponseError as e:
+                    process_storage_error(e)
 
                 # Set the download size to empty
                 self.size = 0
@@ -350,10 +350,11 @@ class StorageStreamDownloader(object):  # pylint: disable=too-many-instance-attr
         self._etag = response.properties.etag
         return response
 
-    def chunks(self):
-        # type: () -> Iterator[bytes]
-        """Iterate over chunks in the download stream.
+    def chunks(self) -> Iterator[bytes]:
+        """
+        Iterate over chunks in the download stream.
 
+        :return: An iterator of the chunks in the download stream.
         :rtype: Iterator[bytes]
         """
         if self.size == 0 or self._download_complete:
@@ -383,18 +384,18 @@ class StorageStreamDownloader(object):  # pylint: disable=too-many-instance-attr
             downloader=iter_downloader,
             chunk_size=self._config.max_chunk_get_size)
 
-    def readall(self):
-        # type: () -> bytes
+    def readall(self) -> bytes:
         """Download the contents of this file.
 
         This operation is blocking until all data is downloaded.
+        :return: The entire blob content as bytes.
         :rtype: bytes
         """
         stream = BytesIO()
         self.readinto(stream)
         data = stream.getvalue()
         if self._encoding:
-            return data.decode(self._encoding)
+            return data.decode(self._encoding)  # type: ignore [return-value]
         return data
 
     def content_as_bytes(self, max_concurrency=1):
@@ -404,8 +405,9 @@ class StorageStreamDownloader(object):  # pylint: disable=too-many-instance-attr
 
         This method is deprecated, use func:`readall` instead.
 
-        :keyword int max_concurrency:
+        :param int max_concurrency:
             The number of parallel connections with which to download.
+        :return: The contents of the file as bytes.
         :rtype: bytes
         """
         warnings.warn(
@@ -422,10 +424,11 @@ class StorageStreamDownloader(object):  # pylint: disable=too-many-instance-attr
 
         This method is deprecated, use func:`readall` instead.
 
-        :keyword int max_concurrency:
+        :param int max_concurrency:
             The number of parallel connections with which to download.
         :param str encoding:
             Test encoding to decode the downloaded bytes. Default is UTF-8.
+        :return: The contents of the file as a str.
         :rtype: str
         """
         warnings.warn(
@@ -436,10 +439,10 @@ class StorageStreamDownloader(object):  # pylint: disable=too-many-instance-attr
         self._encoding = encoding
         return self.readall()
 
-    def readinto(self, stream):
+    def readinto(self, stream: IO[bytes]) -> int:
         """Download the contents of this file to a stream.
 
-        :param stream:
+        :param IO[bytes] stream:
             The stream to download to. This can be an open file-handle,
             or any writable stream. The stream must be seekable if the download
             uses more than one parallel connection.
@@ -455,8 +458,8 @@ class StorageStreamDownloader(object):  # pylint: disable=too-many-instance-attr
 
             try:
                 stream.seek(stream.tell())
-            except (NotImplementedError, AttributeError):
-                raise ValueError(error_message)
+            except (NotImplementedError, AttributeError) as exc:
+                raise ValueError(error_message) from exc
 
         # Write the content to the user stream
         stream.write(self._current_content)
@@ -503,10 +506,12 @@ class StorageStreamDownloader(object):  # pylint: disable=too-many-instance-attr
 
         This method is deprecated, use func:`readinto` instead.
 
-        :param stream:
+        :param IO stream:
             The stream to download to. This can be an open file-handle,
             or any writable stream. The stream must be seekable if the download
             uses more than one parallel connection.
+        :param int max_concurrency:
+            The number of parallel connections with which to download.
         :returns: The properties of the downloaded file.
         :rtype: Any
         """
